﻿using System;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Configuration;
using System.Net.Http;
using System.IO;
using Seagull.BarTender.Print;
using Newtonsoft.Json;

namespace LabelRe_Printer
{
    public partial class FormMain : Form
    {
        private string Host = ConfigurationManager.AppSettings["Host"];
        private string Printer = ConfigurationManager.AppSettings["Printer"];
        private Engine engine;
        private LabelFormatDocument label;
        private Result result;
        private GetGift DataGift;
        private GetSN DataSN;
        private GetSKU DataSKU;
        private string DataInput = "";
        private string GiftID = "";
        private bool Printing;

        public FormMain()
        {
            InitializeComponent();
        }

        private async Task<string[]> GetInfo(string key)
        {
            string status;
            string result;
            using (HttpClient client = new HttpClient())
            {
                var message = await client.GetAsync(key);
                status = message.StatusCode.ToString();
                result = await message.Content.ReadAsStringAsync();
            }
            return new string[] { status, result };
        }

        private void InfoGift()
        {
            try
            {
                var t = Task.Run(() => GetInfo(Host + "/shopfloor/gift/?GiftId=" + GiftID));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    DataGift = JsonConvert.DeserializeObject<GetGift>(t.Result[1]);
                    InfoSN();
                }
                else
                {
                    Printing = false;
                }
            }
            catch
            {
                InfoGift();
            }
        }

        private void InfoSN()
        {
            try
            {
                var t = Task.Run(() => GetInfo(Host + "/shopfloor/productsn/?serialnumber=" + DataGift.results[0].SerialNumber));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    DataSN = JsonConvert.DeserializeObject<GetSN>(t.Result[1]);
                    InfoSKU();
                }
                else
                {
                    Printing = false;
                }
            }
            catch
            {
                InfoSN();
            }
        }

        private void InfoSKU()
        {
            try
            {
                var t = Task.Run(() => GetInfo(Host + "/shopfloor/products/?sku_code=" + DataSN.results[0].SkuCode));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    DataSKU = JsonConvert.DeserializeObject<GetSKU>(t.Result[1]);
                    if (DataSKU.results[0].Family.Equals("400"))
                    {
                        PrintLabel();
                    }
                    else
                    {
                        MessageBox.Show("Model Này Không Được Hỗ Trợ!");
                    }
                }
                else
                {
                    Printing = false;
                }
            }
            catch
            {
                InfoSKU();
            }
        }

        private void PrintLabel()
        {
            try
            {
                label.SubStrings["lblSN"].Value = GiftID;
                label.SubStrings["lblModelName"].Value = DataSKU.results[0].ModelName;
                label.SubStrings["lblCodeAffiliate"].Value = DataSKU.results[0].CodeAffiliate;
                label.SubStrings["lblCodeUPC"].Value = DataSKU.results[0].CodeUPC;
            }
            catch { }
            finally
            {
                result = label.Print();
                if ((int)result == 2)
                {
                    MessageBox.Show("Lỗi Trong Quá Trình In Tem\r\nHãy Khởi Động Lại Máy Tính & Máy In!");
                }
                Printing = false;
            }
        }

        private void First_Setup(object obj, EventArgs arg)
        {
            CheckForIllegalCrossThreadCalls = false;
            try
            {
                engine = new Engine(true);
                label = engine.Documents.Open(Directory.GetCurrentDirectory() + "\\Label.btw");
                engine.Window.Visible = false;
                label.PrintSetup.PrinterName = Printer;
                label.PrintSetup.IdenticalCopiesOfLabel = 1;
            }
            catch
            {
                MessageBox.Show("Hãy Kiểm Tra Lại File Tem Hoặc Tên Máy In!");
            }
        }

        private void Key_Pressed(object obj, KeyPressEventArgs arg)
        {
            if (!Printing)
            {
                if (arg.KeyChar >= 48 && arg.KeyChar <= 57 || arg.KeyChar >= 65 && arg.KeyChar <= 90 || arg.KeyChar >= 97 && arg.KeyChar <= 122)
                {
                    DataInput += arg.KeyChar;
                }
                if (arg.KeyChar == (char)Keys.Enter)
                {
                    DataInput = DataInput.ToUpper();
                    if (DataInput.Length > 9)
                    {
                        Printing = true;
                        GiftID = DataInput;
                        labelScan.Text = GiftID;
                        InfoGift();
                    }
                    DataInput = "";
                }
            }
        }

        private void Finish(object obj, FormClosingEventArgs arg)
        {
            try
            {
                label.Close(SaveOptions.DoNotSaveChanges);
                engine.Stop();
            }
            catch { }
        }
    }

    public class GetGift
    {
        public ResultGift[] results { get; set; }
    }

    public class ResultGift
    {
        public string SerialNumber { get; set; }
    }

    public class GetSN
    {
        public ResultSN[] results { get; set; }
    }

    public class ResultSN
    {
        public string SkuCode { get; set; }
    }

    public class GetSKU
    {
        public ResultSKU[] results { get; set; }
    }

    public class ResultSKU
    {
        public string Family { get; set; }
        public string ModelName { get; set; }
        public string CodeUPC { get; set; }
        public string CodeAffiliate { get; set; }
    }
}
